@@ -267,7 +267,9 @@ same_key (const Key *key, const Key *other)
                                      if (*c1 != *c2)
                                         return FALSE;
                         }
-                }
+                } else if (key->keycodes != NULL || other->keycodes != NULL)
+                        return FALSE;
+
 
                 return TRUE;
         }
@@ -413,7 +415,7 @@ get_exec_environment (XEvent *xevent)
         GdkWindow *window = gdk_xid_table_lookup (xevent->xkey.root);
 
         if (window) {
-                screen = gdk_drawable_get_screen (GDK_DRAWABLE (window));
+                screen = gdk_window_get_screen (window);
         }
 
         g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
@@ -565,15 +567,16 @@ gsd_keybindings_manager_start (GsdKeybindingsManager *manager,
 
         client = gconf_client_get_default ();
 
-        manager->priv->allowed_keys = gconf_client_get_list (client,
-                                                             ALLOWED_KEYS_KEY,
-                                                             GCONF_VALUE_STRING,
-                                                             NULL);
         manager->priv->notify = register_config_callback (manager,
                                                           client,
                                                           GCONF_BINDING_DIR,
                                                           (GConfClientNotifyFunc) bindings_callback);
 
+        manager->priv->allowed_keys = gconf_client_get_list (client,
+                                                             ALLOWED_KEYS_KEY,
+                                                             GCONF_VALUE_STRING,
+                                                             NULL);
+
         dpy = gdk_display_get_default ();
         screen_num = gdk_display_get_n_screens (dpy);
 
