@@ -36,12 +36,14 @@
 #include <libgnomekbd/gkbd-keyboard-config.h>
 #include <libgnomekbd/gkbd-util.h>
 
-#include "gsd-xmodmap.h"
 #include "gsd-keyboard-xkb.h"
 #include "delayed-dialog.h"
 #include "gnome-settings-profile.h"
 
-#define GTK_RESPONSE_PRINT 2
+#define SETTINGS_KEYBOARD_DIR "org.gnome.settings-daemon.plugins.keyboard"
+
+#define DISABLE_INDICATOR_KEY "disable-indicator"
+#define SHOW_KEYBOARD_LEDS_INDICATOR_KEY "show-keyboard-leds-indicator"
 
 static GsdKeyboardManager *manager = NULL;
 
@@ -56,21 +58,13 @@ static GkbdKeyboardConfig initial_sys_kbd_config;
 
 static gboolean inited_ok = FALSE;
 
+static GSettings *settings_plugin = NULL;
 static GSettings *settings_desktop = NULL;
 static GSettings *settings_keyboard = NULL;
 
 static PostActivationCallback pa_callback = NULL;
 static void *pa_callback_user_data = NULL;
 
-static const char KNOWN_FILES_KEY[] =
-    "/desktop/gnome/peripherals/keyboard/general/known_file_list";
-
-static const char DISABLE_INDICATOR_KEY[] =
-    "/desktop/gnome/peripherals/keyboard/general/disable_indicator";
-
-static const char DUPLICATE_LEDS_KEY[] =
-    "/desktop/gnome/peripherals/keyboard/general/duplicate_leds";
-
 static const char *gdm_keyboard_layout = NULL;
 
 static GtkStatusIcon *icon = NULL;
@@ -94,39 +88,13 @@ static const gchar *indicator_off_icon_names[] = {
 	"kbd-capslock-off"
 };
 
-#define noGSDKX
-
-#ifdef GSDKX
-static FILE *logfile;
-
-static void
-gsd_keyboard_log_appender (const char file[],
-			   const char function[],
-			   int level, const char format[], va_list args)
-{
-	time_t now = time (NULL);
-	fprintf (logfile, "[%08ld,%03d,%s:%s/] \t", now,
-		 level, file, function);
-	vfprintf (logfile, format, args);
-	fflush (logfile);
-}
-#endif
-
 static void
 activation_error (void)
 {
-	char const *vendor =
-	    ServerVendor (GDK_DISPLAY_XDISPLAY
-			  (gdk_display_get_default ()));
-	int release =
-	    VendorRelease (GDK_DISPLAY_XDISPLAY
-			   (gdk_display_get_default ()));
+	char const *vendor;
 	GtkWidget *dialog;
-	gboolean badXFree430Release;
 
-	badXFree430Release = (vendor != NULL)
-	    && (0 == strcmp (vendor, "The XFree86 Project, Inc"))
-	    && (release / 100000 == 403);
+	vendor = ServerVendor (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
 
 	/* VNC viewers will not work, do not barrage them with warnings */
 	if (NULL != vendor && NULL != strstr (vendor, "VNC"))
@@ -138,25 +106,16 @@ activation_error (void)
 						     GTK_BUTTONS_CLOSE,
 						     _
 						     ("Error activating XKB configuration.\n"
-						      "It can happen under various circumstances:\n"
-						      " • a bug in libxklavier library\n"
-						      " • a bug in X server (xkbcomp, xmodmap utilities)\n"
-						      " • X server with incompatible libxkbfile implementation\n\n"
-						      "X server version data:\n%s\n%d\n%s\n"
-						      "If you report this situation as a bug, please include:\n"
-						      " • The result of <b>%s</b>\n"
-						      " • The result of <b>%s</b>"),
-						     vendor,
-						     release,
-						     badXFree430Release
-						     ?
-						     _
-						     ("You are using XFree 4.3.0.\n"
-						      "There are known problems with complex XKB configurations.\n"
-						      "Try using a simpler configuration or using a later version of the XFree software.")
-						     : "",
+						      "There can be various reasons for that.\n\n"
+						      "If you report this situation as a bug, include the results of\n"
+						      " • <b>%s</b>\n"
+						      " • <b>%s</b>\n"
+						      " • <b>%s</b>\n"
+						      " • <b>%s</b>"),
 						     "xprop -root | grep XKB",
-						     "gconftool-2 -R /desktop/gnome/peripherals/keyboard/kbd");
+						     "gsettings get org.gnome.libgnomekbd.keyboard model",
+						     "gsettings get org.gnome.libgnomekbd.keyboard layouts",
+						     "gsettings get org.gnome.libgnomekbd.keyboard options");
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy), NULL);
 	gsd_delayed_show_dialog (dialog);
@@ -165,7 +124,6 @@ activation_error (void)
 static void
 apply_desktop_settings (void)
 {
-	GConfClient *conf_client;
 	gboolean show_leds;
 	int i;
 	if (!inited_ok)
@@ -177,10 +135,7 @@ apply_desktop_settings (void)
 	   before activating them */
 	gkbd_desktop_config_activate (&current_config);
 
-	conf_client = gconf_client_get_default ();
-	show_leds =
-	    gconf_client_get_bool (conf_client, DUPLICATE_LEDS_KEY, NULL);
-	g_object_unref (conf_client);
+	show_leds = g_settings_get_boolean (settings_plugin, SHOW_KEYBOARD_LEDS_INDICATOR_KEY);
 	for (i = sizeof (indicator_icons) / sizeof (indicator_icons[0]);
 	     --i >= 0;) {
 		gtk_status_icon_set_visible (indicator_icons[i],
@@ -339,13 +294,9 @@ show_hide_icon ()
 {
 	if (g_strv_length (current_kbd_config.layouts_variants) > 1) {
 		if (icon == NULL) {
-			GConfClient *conf_client =
-			    gconf_client_get_default ();
 			gboolean disable =
-			    gconf_client_get_bool (conf_client,
-						   DISABLE_INDICATOR_KEY,
-						   NULL);
-			g_object_unref (conf_client);
+			    g_settings_get_boolean (settings_plugin,
+						    DISABLE_INDICATOR_KEY);
 			if (disable)
 				return;
 
@@ -411,20 +362,15 @@ filter_xkb_config (void)
 	while (*lv) {
 		xkl_debug (100, "Checking [%s]\n", *lv);
 		if (gkbd_keyboard_config_split_items (*lv, &lname, &vname)) {
+			gboolean should_be_dropped = FALSE;
 			g_snprintf (item->name, sizeof (item->name), "%s",
 				    lname);
 			if (!xkl_config_registry_find_layout
 			    (xkl_registry, item)) {
 				xkl_debug (100, "Bad layout [%s]\n",
 					   lname);
-				g_free (*lv);
-				memmove (lv, lv + 1,
-					 g_strv_length (lv) *
-					 sizeof (gchar *));
-				any_change = TRUE;
-				continue;
-			}
-			if (vname) {
+				should_be_dropped = TRUE;
+			} else if (vname) {
 				g_snprintf (item->name,
 					    sizeof (item->name), "%s",
 					    vname);
@@ -433,14 +379,14 @@ filter_xkb_config (void)
 					xkl_debug (100,
 						   "Bad variant [%s(%s)]\n",
 						   lname, vname);
-					g_free (*lv);
-					memmove (lv, lv + 1,
-						 g_strv_length (lv) *
-						 sizeof (gchar *));
-					any_change = TRUE;
-					continue;
+					should_be_dropped = TRUE;
 				}
 			}
+			if (should_be_dropped) {
+				gkbd_strv_behead (lv);
+				any_change = TRUE;
+				continue;
+			}
 		}
 		lv++;
 	}
@@ -451,7 +397,6 @@ filter_xkb_config (void)
 static void
 apply_xkb_settings (void)
 {
-	GConfClient *conf_client;
 	GkbdKeyboardConfig current_sys_kbd_config;
 	int group_to_activate = -1;
 	char *gdm_layout;
@@ -460,18 +405,16 @@ apply_xkb_settings (void)
 	if (!inited_ok)
 		return;
 
-	conf_client = gconf_client_get_default ();
-
 	/* With GDM the user can already set a layout from the login
 	 * screen. Try to keep that setting.
 	 * We clear gdm_keyboard_layout early, so we don't risk
-	 * recursion from gconf notification.
+	 * recursion from conf notification.
 	 */
 	gdm_layout = g_strdup (gdm_keyboard_layout);
 	gdm_keyboard_layout = NULL;
 
 	/* gdm's configuration and $GDM_KEYBOARD_LAYOUT separates layout and
-	 * variant with a space, but gconf uses tabs; so convert to be robust
+	 * variant with a space, but gsettings use tabs; so convert to be robust
 	 * with both */
 	for (s = gdm_layout; s && *s; ++s) {
 		if (*s == ' ') {
@@ -497,8 +440,8 @@ apply_xkb_settings (void)
 			int len;
 
 			i = layouts =
-			    g_strdupv (initial_sys_kbd_config.
-				       layouts_variants);
+			    g_strdupv
+			    (initial_sys_kbd_config.layouts_variants);
 			if (i != NULL) {
 				while (*i != NULL) {
 					s = *i;
@@ -538,13 +481,10 @@ apply_xkb_settings (void)
 				layouts[max_groups - 1] =
 				    g_strdup (gdm_layout);
 			} else {
-				gchar **nl =
-				    g_new0 (gchar *, old_length + 2);
-				memcpy (nl, layouts,
-					old_length * sizeof (gchar *));
-				nl[old_length] = g_strdup (gdm_layout);
-				g_free (layouts);
-				layouts = nl;
+				layouts =
+				    gkbd_strv_append (layouts,
+						      g_strdup
+						      (gdm_layout));
 			}
 
 			g_settings_set_strv (settings_keyboard,
@@ -625,83 +565,6 @@ gsd_keyboard_xkb_analyze_sysconfig (void)
 						  NULL);
 }
 
-static gboolean
-gsd_chk_file_list (void)
-{
-	GDir *home_dir;
-	const char *fname;
-	GSList *file_list = NULL;
-	GSList *last_login_file_list = NULL;
-	GSList *tmp = NULL;
-	GSList *tmp_l = NULL;
-	gboolean new_file_exist = FALSE;
-	GConfClient *conf_client;
-
-	home_dir = g_dir_open (g_get_home_dir (), 0, NULL);
-	while ((fname = g_dir_read_name (home_dir)) != NULL) {
-		if (g_strrstr (fname, "modmap")) {
-			file_list =
-			    g_slist_append (file_list, g_strdup (fname));
-		}
-	}
-	g_dir_close (home_dir);
-
-	conf_client = gconf_client_get_default ();
-
-	last_login_file_list = gconf_client_get_list (conf_client,
-						      KNOWN_FILES_KEY,
-						      GCONF_VALUE_STRING,
-						      NULL);
-
-	/* Compare between the two file list, currently available modmap files
-	   and the files available in the last log in */
-	tmp = file_list;
-	while (tmp != NULL) {
-		tmp_l = last_login_file_list;
-		new_file_exist = TRUE;
-		while (tmp_l != NULL) {
-			if (strcmp (tmp->data, tmp_l->data) == 0) {
-				new_file_exist = FALSE;
-				break;
-			} else {
-				tmp_l = tmp_l->next;
-			}
-		}
-		if (new_file_exist) {
-			break;
-		} else {
-			tmp = tmp->next;
-		}
-	}
-
-	if (new_file_exist) {
-		gconf_client_set_list (conf_client,
-				       KNOWN_FILES_KEY,
-				       GCONF_VALUE_STRING,
-				       file_list, NULL);
-	}
-
-	g_object_unref (conf_client);
-
-	g_slist_foreach (file_list, (GFunc) g_free, NULL);
-	g_slist_free (file_list);
-
-	g_slist_foreach (last_login_file_list, (GFunc) g_free, NULL);
-	g_slist_free (last_login_file_list);
-
-	return new_file_exist;
-
-}
-
-static void
-gsd_keyboard_xkb_chk_lcl_xmm (void)
-{
-	if (gsd_chk_file_list ()) {
-		gsd_modmap_dialog_call ();
-	}
-	gsd_load_modmap_files ();
-}
-
 void
 gsd_keyboard_xkb_set_post_activation_callback (PostActivationCallback fun,
 					       void *user_data)
@@ -778,6 +641,8 @@ gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 	    GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 	gnome_settings_profile_start (NULL);
 
+	settings_plugin = g_settings_new (SETTINGS_KEYBOARD_DIR);
+
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 					   DATADIR G_DIR_SEPARATOR_S
 					   "icons");
@@ -795,11 +660,6 @@ gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 
 	gsd_keyboard_update_indicator_icons ();
 
-#ifdef GSDKX
-	xkl_set_debug_level (200);
-	logfile = fopen ("/tmp/gsdkx.log", "a");
-	xkl_set_log_appender (gsd_keyboard_log_appender);
-#endif
 	manager = kbd_manager;
 	gnome_settings_profile_start ("xkl_engine_get_instance");
 	xkl_engine = xkl_engine_get_instance (display);
@@ -814,11 +674,6 @@ gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 					   xkl_engine);
 		xkl_engine_backup_names_prop (xkl_engine);
 		gsd_keyboard_xkb_analyze_sysconfig ();
-		gnome_settings_profile_start
-		    ("gsd_keyboard_xkb_chk_lcl_xmm");
-		gsd_keyboard_xkb_chk_lcl_xmm ();
-		gnome_settings_profile_end
-		    ("gsd_keyboard_xkb_chk_lcl_xmm");
 
 		settings_desktop = g_settings_new (GKBD_DESKTOP_SCHEMA);
 		settings_keyboard = g_settings_new (GKBD_KEYBOARD_SCHEMA);
@@ -889,6 +744,8 @@ gsd_keyboard_xkb_shutdown (void)
 	settings_desktop = NULL;
 	g_object_unref (settings_keyboard);
 	settings_keyboard = NULL;
+	g_object_unref (settings_plugin);
+	settings_plugin = NULL;
 
 	if (xkl_registry) {
 		g_object_unref (xkl_registry);
