@@ -33,6 +33,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -40,57 +41,57 @@
 #include <X11/keysym.h>
 #include <X11/Xatom.h>
 
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 #include <X11/extensions/XInput.h>
 #include <X11/extensions/XIproto.h>
-#endif
-#include <gconf/gconf.h>
-#include <gconf/gconf-client.h>
 
 #include "gnome-settings-profile.h"
 #include "gsd-mouse-manager.h"
+#include "gsd-input-helper.h"
+#include "gsd-enums.h"
 
 #define GSD_MOUSE_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_MOUSE_MANAGER, GsdMouseManagerPrivate))
 
-#define GCONF_MOUSE_DIR         "/desktop/gnome/peripherals/mouse"
-#define GCONF_MOUSE_A11Y_DIR    "/desktop/gnome/accessibility/mouse"
-#define GCONF_TOUCHPAD_DIR      "/desktop/gnome/peripherals/touchpad"
+#define SETTINGS_MOUSE_DIR         "org.gnome.settings-daemon.peripherals.mouse"
+#define SETTINGS_TOUCHPAD_DIR      "org.gnome.settings-daemon.peripherals.touchpad"
 
-#define KEY_LEFT_HANDED         GCONF_MOUSE_DIR "/left_handed"
-#define KEY_MOTION_ACCELERATION GCONF_MOUSE_DIR "/motion_acceleration"
-#define KEY_MOTION_THRESHOLD    GCONF_MOUSE_DIR "/motion_threshold"
-#define KEY_LOCATE_POINTER      GCONF_MOUSE_DIR "/locate_pointer"
-#define KEY_DWELL_ENABLE        GCONF_MOUSE_A11Y_DIR "/dwell_enable"
-#define KEY_DELAY_ENABLE        GCONF_MOUSE_A11Y_DIR "/delay_enable"
-#define KEY_TOUCHPAD_DISABLE_W_TYPING    GCONF_TOUCHPAD_DIR "/disable_while_typing"
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
-#define KEY_TAP_TO_CLICK        GCONF_TOUCHPAD_DIR "/tap_to_click"
-#define KEY_SCROLL_METHOD       GCONF_TOUCHPAD_DIR "/scroll_method"
-#define KEY_PAD_HORIZ_SCROLL    GCONF_TOUCHPAD_DIR "/horiz_scroll_enabled"
-#define KEY_TOUCHPAD_ENABLED    GCONF_TOUCHPAD_DIR "/touchpad_enabled"
-#endif
+/* Keys for both touchpad and mouse */
+#define KEY_LEFT_HANDED         "left-handed"                /* a boolean for mouse, an enum for touchpad */
+#define KEY_MOTION_ACCELERATION "motion-acceleration"
+#define KEY_MOTION_THRESHOLD    "motion-threshold"
+
+/* Touchpad settings */
+#define KEY_TOUCHPAD_DISABLE_W_TYPING    "disable-while-typing"
+#define KEY_PAD_HORIZ_SCROLL             "horiz-scroll-enabled"
+#define KEY_SCROLL_METHOD                "scroll-method"
+#define KEY_TAP_TO_CLICK                 "tap-to-click"
+#define KEY_TOUCHPAD_ENABLED             "touchpad-enabled"
+
+/* Mouse settings */
+#define KEY_LOCATE_POINTER               "locate-pointer"
+#define KEY_DWELL_CLICK_ENABLED          "dwell-click-enabled"
+#define KEY_SECONDARY_CLICK_ENABLED      "secondary-click-enabled"
+#define KEY_MIDDLE_BUTTON_EMULATION      "middle-button-enabled"
 
 struct GsdMouseManagerPrivate
 {
+        GSettings *touchpad_settings;
+        GSettings *mouse_settings;
+	GSettings *mouse_a11y_settings;
+        GdkDeviceManager *device_manager;
         guint notify;
-        guint notify_a11y;
-        guint notify_touchpad;
 
         gboolean mousetweaks_daemon_running;
         gboolean syndaemon_spawned;
         GPid syndaemon_pid;
-	gboolean locate_pointer_spawned;
-	GPid locate_pointer_pid;
+        gboolean locate_pointer_spawned;
+        GPid locate_pointer_pid;
 };
 
 static void     gsd_mouse_manager_class_init  (GsdMouseManagerClass *klass);
 static void     gsd_mouse_manager_init        (GsdMouseManager      *mouse_manager);
 static void     gsd_mouse_manager_finalize    (GObject             *object);
 static void     set_mouse_settings            (GsdMouseManager      *manager);
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static int      set_tap_to_click              (gboolean state, gboolean left_handed);
-static XDevice* device_is_touchpad            (XDeviceInfo deviceinfo);
-#endif
 
 G_DEFINE_TYPE (GsdMouseManager, gsd_mouse_manager, G_TYPE_OBJECT)
 
@@ -172,7 +173,6 @@ gsd_mouse_manager_class_init (GsdMouseManagerClass *klass)
 }
 
 
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static gboolean
 supports_xinput_devices (void)
 {
@@ -184,7 +184,6 @@ supports_xinput_devices (void)
                                 &event,
                                 &error);
 }
-#endif
 
 static void
 configure_button_layout (guchar   *buttons,
@@ -243,7 +242,6 @@ configure_button_layout (guchar   *buttons,
         }
 }
 
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static gboolean
 xinput_device_has_buttons (XDeviceInfo *device_info)
 {
@@ -297,7 +295,9 @@ touchpad_has_single_button (XDevice *device)
 
 
 static void
-set_xinput_devices_left_handed (gboolean left_handed)
+set_xinput_devices_left_handed (GsdMouseManager *manager,
+                                gboolean mouse_left_handed,
+                                gboolean touchpad_left_handed)
 {
         XDeviceInfo *device_info;
         gint n_devices;
@@ -315,10 +315,11 @@ set_xinput_devices_left_handed (gboolean left_handed)
 
         for (i = 0; i < n_devices; i++) {
                 XDevice *device = NULL;
+                gboolean left_handed;
 
                 if ((device_info[i].use == IsXPointer) ||
                     (device_info[i].use == IsXKeyboard) ||
-                    (device_info[i].name != NULL && strcmp ("Virtual core XTEST pointer", device_info[i].name) == 0) ||
+                    (device_info[i].name != NULL && g_str_equal ("Virtual core XTEST pointer", device_info[i].name)) ||
                     (!xinput_device_has_buttons (&device_info[i])))
                         continue;
 
@@ -326,17 +327,19 @@ set_xinput_devices_left_handed (gboolean left_handed)
                  * around too, otherwise a tap would be a right-click */
                 device = device_is_touchpad (device_info[i]);
                 if (device != NULL) {
-                        GConfClient *client = gconf_client_get_default ();
-                        gboolean tap = gconf_client_get_bool (client, KEY_TAP_TO_CLICK, NULL);
+                        gboolean tap = g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TAP_TO_CLICK);
                         gboolean single_button = touchpad_has_single_button (device);
 
+                        left_handed = touchpad_left_handed;
+
                         if (tap && !single_button)
                                 set_tap_to_click (tap, left_handed);
                         XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
-                        g_object_unref (client);
 
                         if (single_button)
-                            continue;
+                                continue;
+                } else {
+                        left_handed = mouse_left_handed;
                 }
 
                 gdk_error_trap_push ();
@@ -372,67 +375,47 @@ set_xinput_devices_left_handed (gboolean left_handed)
                 XFreeDeviceList (device_info);
 }
 
-static GdkFilterReturn
-devicepresence_filter (GdkXEvent *xevent,
-                       GdkEvent  *event,
-                       gpointer   data)
+static void
+device_added_cb (GdkDeviceManager *device_manager,
+                 GdkDevice        *device,
+                 gpointer          user_data)
 {
-        XEvent *xev = (XEvent *) xevent;
-        XEventClass class_presence;
-        int xi_presence;
-
-        DevicePresence (gdk_x11_get_default_xdisplay (), xi_presence, class_presence);
-
-        if (xev->type == xi_presence)
-        {
-            XDevicePresenceNotifyEvent *dpn = (XDevicePresenceNotifyEvent *) xev;
-            if (dpn->devchange == DeviceEnabled)
-                set_mouse_settings ((GsdMouseManager *) data);
-        }
-        return GDK_FILTER_CONTINUE;
+        if (gdk_device_get_source (device) == GDK_SOURCE_MOUSE)
+                set_mouse_settings ((GsdMouseManager *) user_data);
 }
 
 static void
 set_devicepresence_handler (GsdMouseManager *manager)
 {
-        Display *display;
-        XEventClass class_presence;
-        int xi_presence;
+        GdkDeviceManager *device_manager;
 
-        if (!supports_xinput_devices ())
+        device_manager = gdk_display_get_device_manager (gdk_display_get_default ());
+        if (device_manager == NULL)
                 return;
 
-        display = gdk_x11_get_default_xdisplay ();
-
-        gdk_error_trap_push ();
-        DevicePresence (display, xi_presence, class_presence);
-        XSelectExtensionEvent (display,
-                               RootWindow (display, DefaultScreen (display)),
-                               &class_presence, 1);
-
-        gdk_flush ();
-        if (!gdk_error_trap_pop ())
-                gdk_window_add_filter (NULL, devicepresence_filter, manager);
+        g_signal_connect (G_OBJECT (device_manager), "device-added",
+                          G_CALLBACK (device_added_cb), manager);
+        manager->priv->device_manager = device_manager;
 }
-#endif
 
 static void
 set_left_handed (GsdMouseManager *manager,
-                 gboolean         left_handed)
+                 gboolean         mouse_left_handed,
+                 gboolean         touchpad_left_handed)
 {
         guchar *buttons ;
         gsize buttons_capacity = 16;
         gint n_buttons, i;
 
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         if (supports_xinput_devices ()) {
                 /* When XInput support is available, never set the
                  * button ordering on the core pointer as that would
                  * revert the changes we make on the devices themselves */
-                set_xinput_devices_left_handed (left_handed);
+                set_xinput_devices_left_handed (manager,
+                                                mouse_left_handed,
+                                                touchpad_left_handed);
                 return;
         }
-#endif
 
         buttons = g_new (guchar, buttons_capacity);
         n_buttons = XGetPointerMapping (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
@@ -448,7 +431,7 @@ set_left_handed (GsdMouseManager *manager,
                                                 (gint) buttons_capacity);
         }
 
-        configure_button_layout (buttons, n_buttons, left_handed);
+        configure_button_layout (buttons, n_buttons, mouse_left_handed);
 
         /* X refuses to change the mapping while buttons are engaged,
          * so if this is the case we'll retry a few times
@@ -506,48 +489,71 @@ set_motion_threshold (GsdMouseManager *manager,
                                0, 0, motion_threshold);
 }
 
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
-static XDevice*
-device_is_touchpad (XDeviceInfo deviceinfo)
+static void
+set_middle_button (GsdMouseManager *manager,
+                   gboolean         middle_button)
 {
-        XDevice *device;
-        Atom realtype, prop;
-        int realformat;
-        unsigned long nitems, bytes_after;
-        unsigned char *data;
+        XDeviceInfo *device_info;
+        gint n_devices;
+        gint i;
+        Atom prop;
 
-        if (deviceinfo.type != XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), XI_TOUCHPAD, False))
-                return NULL;
+        prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                            "Evdev Middle Button Emulation", True);
 
-        prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Off", False);
-        if (!prop)
-                return NULL;
+        if (!prop) /* no evdev devices */
+                return;
 
-        gdk_error_trap_push ();
-        device = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), deviceinfo.id);
-        if (gdk_error_trap_pop () || (device == NULL))
-                return NULL;
+        device_info = XListInputDevices (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &n_devices);
+
+        for (i = 0; i < n_devices; i++) {
+                XDevice *device = NULL;
+                Atom type;
+                int format;
+                unsigned long nitems, bytes_after;
+                unsigned char *data;
+
+                gdk_error_trap_push ();
+
+                device = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device_info[i].id);
+
+                if ((gdk_error_trap_pop () != 0) ||
+                    (device == NULL))
+                        continue;
+
+                gdk_error_trap_push ();
+
+                XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                    device, prop, 0, 1, False, XA_INTEGER, &type, &format,
+                                    &nitems, &bytes_after, &data);
+
+                if ((gdk_error_trap_pop () != 0)) {
+                        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+                        continue;
+                }
+
+                if (format == 8 && type == XA_INTEGER && nitems == 1) {
+                        data[0] = middle_button ? 1 : 0;
+
+                        gdk_error_trap_push ();
+                        XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                               device, prop, type, format, PropModeReplace, data, nitems);
+
+                        gdk_error_trap_pop ();
+                }
 
-        gdk_error_trap_push ();
-        if ((XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device, prop, 0, 1, False,
-                                XA_INTEGER, &realtype, &realformat, &nitems,
-                                &bytes_after, &data) == Success) && (realtype != None)) {
-                gdk_error_trap_pop ();
                 XFree (data);
-                return device;
+                XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
         }
-        gdk_error_trap_pop ();
 
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
-        return NULL;
+        if (device_info != NULL)
+                XFreeDeviceList (device_info);
 }
-#endif
 
 static int
 set_disable_w_typing (GsdMouseManager *manager, gboolean state)
 {
-
-        if (state) {
+        if (state && touchpad_is_present ()) {
                 GError *error = NULL;
                 char *args[5];
 
@@ -570,15 +576,10 @@ set_disable_w_typing (GsdMouseManager *manager, gboolean state)
                 manager->priv->syndaemon_spawned = (error == NULL);
 
                 if (error) {
-                        GConfClient *client;
-                        client = gconf_client_get_default ();
-                        gconf_client_set_bool (client, KEY_TOUCHPAD_DISABLE_W_TYPING, FALSE, NULL);
-                        g_object_unref (client);
+                        g_settings_set_boolean (manager->priv->touchpad_settings, KEY_TOUCHPAD_DISABLE_W_TYPING, FALSE);
                         g_error_free (error);
                 }
-
-        } else if (manager->priv->syndaemon_spawned)
-        {
+        } else if (manager->priv->syndaemon_spawned) {
                 kill (manager->priv->syndaemon_pid, SIGHUP);
                 g_spawn_close_pid (manager->priv->syndaemon_pid);
                 manager->priv->syndaemon_spawned = FALSE;
@@ -587,7 +588,6 @@ set_disable_w_typing (GsdMouseManager *manager, gboolean state)
         return 0;
 }
 
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static int
 set_tap_to_click (gboolean state, gboolean left_handed)
 {
@@ -613,8 +613,7 @@ set_tap_to_click (gboolean state, gboolean left_handed)
                                                 False, XA_INTEGER, &type, &format, &nitems,
                                                 &bytes_after, &data);
 
-                        if (rc == Success && type == XA_INTEGER && format == 8 && nitems >= 7)
-                        {
+                        if (rc == Success && type == XA_INTEGER && format == 8 && nitems >= 7) {
                                 /* Set RLM mapping for 1/2/3 fingers*/
                                 data[4] = (state) ? ((left_handed) ? 3 : 1) : 0;
                                 data[5] = (state) ? ((left_handed) ? 1 : 3) : 0;
@@ -700,12 +699,8 @@ set_horiz_scroll (gboolean state)
 }
 
 
-/**
- * Scroll methods are: 0 - disabled, 1 - edge scrolling, 2 - twofinger
- * scrolling
- */
 static int
-set_edge_scroll (int method)
+set_edge_scroll (GsdTouchpadScrollMethod method)
 {
         int numdevices, i, rc;
         XDeviceInfo *devicelist = XListInputDevices (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &numdevices);
@@ -733,7 +728,7 @@ set_edge_scroll (int method)
                                                 &bytes_after, &data);
                         if (rc == Success && act_type == XA_INTEGER &&
                                 act_format == 8 && nitems >= 2) {
-                                data[0] = (method == 1) ? 1 : 0;
+                                data[0] = (method == GSD_TOUCHPAD_SCROLL_METHOD_EDGE_SCROLLING) ? 1 : 0;
                                 XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device,
                                                         prop_edge, XA_INTEGER, 8,
                                                         PropModeReplace, data, nitems);
@@ -747,7 +742,7 @@ set_edge_scroll (int method)
                                                 &bytes_after, &data);
                         if (rc == Success && act_type == XA_INTEGER &&
                                 act_format == 8 && nitems >= 2) {
-                                data[0] = (method == 2) ? 1 : 0;
+                                data[0] = (method == GSD_TOUCHPAD_SCROLL_METHOD_TWO_FINGER_SCROLLING) ? 1 : 0;
                                 XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device,
                                                         prop_twofinger, XA_INTEGER, 8,
                                                         PropModeReplace, data, nitems);
@@ -803,7 +798,6 @@ set_touchpad_enabled (gboolean state)
         XFreeDeviceList (devicelist);
         return 0;
 }
-#endif
 
 static void
 set_locate_pointer (GsdMouseManager *manager,
@@ -826,15 +820,12 @@ set_locate_pointer (GsdMouseManager *manager,
                 manager->priv->locate_pointer_spawned = (error == NULL);
 
                 if (error) {
-                        GConfClient *client;
-                        client = gconf_client_get_default ();
-                        gconf_client_set_bool (client, KEY_LOCATE_POINTER, FALSE, NULL);
-                        g_object_unref (client);
+                        g_settings_set_boolean (manager->priv->mouse_settings, KEY_LOCATE_POINTER, FALSE);
                         g_error_free (error);
                 }
 
         }
-	else if (manager->priv->locate_pointer_spawned) {
+        else if (manager->priv->locate_pointer_spawned) {
                 kill (manager->priv->locate_pointer_pid, SIGHUP);
                 g_spawn_close_pid (manager->priv->locate_pointer_pid);
                 manager->priv->locate_pointer_spawned = FALSE;
@@ -843,12 +834,12 @@ set_locate_pointer (GsdMouseManager *manager,
 
 static void
 set_mousetweaks_daemon (GsdMouseManager *manager,
-                        gboolean         dwell_enable,
-                        gboolean         delay_enable)
+                        gboolean         dwell_click_enabled,
+                        gboolean         secondary_click_enabled)
 {
         GError *error = NULL;
         gchar *comm;
-        gboolean run_daemon = dwell_enable || delay_enable;
+        gboolean run_daemon = dwell_click_enabled || secondary_click_enabled;
 
         if (run_daemon || manager->priv->mousetweaks_daemon_running)
                 comm = g_strdup_printf ("mousetweaks %s",
@@ -859,23 +850,17 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
         if (run_daemon)
                 manager->priv->mousetweaks_daemon_running = TRUE;
 
-
         if (! g_spawn_command_line_async (comm, &error)) {
-                if (error->code == G_SPAWN_ERROR_NOENT &&
-                    (dwell_enable || delay_enable)) {
+                if (error->code == G_SPAWN_ERROR_NOENT && run_daemon) {
                         GtkWidget *dialog;
-                        GConfClient *client;
 
-                        client = gconf_client_get_default ();
-                        if (dwell_enable)
-                                gconf_client_set_bool (client,
-                                                       KEY_DWELL_ENABLE,
-                                                       FALSE, NULL);
-                        else if (delay_enable)
-                                gconf_client_set_bool (client,
-                                                       KEY_DELAY_ENABLE,
-                                                       FALSE, NULL);
-                        g_object_unref (client);
+                        if (dwell_click_enabled) {
+                                g_settings_set_boolean (manager->priv->mouse_a11y_settings,
+                                                        KEY_DWELL_CLICK_ENABLED, FALSE);
+                        } else if (secondary_click_enabled) {
+                                g_settings_set_boolean (manager->priv->mouse_a11y_settings,
+                                                        KEY_SECONDARY_CLICK_ENABLED, FALSE);
+                        }
 
                         dialog = gtk_message_dialog_new (NULL, 0,
                                                          GTK_MESSAGE_WARNING,
@@ -884,10 +869,9 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
                         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                                                   _("Mouse accessibility requires Mousetweaks "
                                                                     "to be installed on your system."));
-                        gtk_window_set_title (GTK_WINDOW (dialog),
-                                              _("Mouse Preferences"));
+                        gtk_window_set_title (GTK_WINDOW (dialog), _("Universal Access"));
                         gtk_window_set_icon_name (GTK_WINDOW (dialog),
-                                                  "input-mouse");
+                                                  "preferences-desktop-accessibility");
                         gtk_dialog_run (GTK_DIALOG (dialog));
                         gtk_widget_destroy (dialog);
                 }
@@ -896,96 +880,85 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
         g_free (comm);
 }
 
-static void
-set_mouse_settings (GsdMouseManager *manager)
+static gboolean
+get_touchpad_handedness (GsdMouseManager *manager, gboolean mouse_left_handed)
 {
-        GConfClient *client = gconf_client_get_default ();
-        gboolean left_handed = gconf_client_get_bool (client, KEY_LEFT_HANDED, NULL);
-
-        set_left_handed (manager, left_handed);
-        set_motion_acceleration (manager, gconf_client_get_float (client, KEY_MOTION_ACCELERATION , NULL));
-        set_motion_threshold (manager, gconf_client_get_int (client, KEY_MOTION_THRESHOLD, NULL));
-
-        set_disable_w_typing (manager, gconf_client_get_bool (client, KEY_TOUCHPAD_DISABLE_W_TYPING, NULL));
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
-        set_tap_to_click (gconf_client_get_bool (client, KEY_TAP_TO_CLICK, NULL), left_handed);
-        set_edge_scroll (gconf_client_get_int (client, KEY_SCROLL_METHOD, NULL));
-        set_horiz_scroll (gconf_client_get_bool (client, KEY_PAD_HORIZ_SCROLL, NULL));
-        set_touchpad_enabled (gconf_client_get_bool (client, KEY_TOUCHPAD_ENABLED, NULL));
-#endif
-
-        g_object_unref (client);
-}
-
-static void
-mouse_callback (GConfClient        *client,
-                guint               cnxn_id,
-                GConfEntry         *entry,
-                GsdMouseManager    *manager)
-{
-        if (! strcmp (entry->key, KEY_LEFT_HANDED)) {
-                if (entry->value->type == GCONF_VALUE_BOOL) {
-                        set_left_handed (manager, gconf_value_get_bool (entry->value));
-                }
-        } else if (! strcmp (entry->key, KEY_MOTION_ACCELERATION)) {
-                if (entry->value->type == GCONF_VALUE_FLOAT) {
-                        set_motion_acceleration (manager, gconf_value_get_float (entry->value));
-                }
-        } else if (! strcmp (entry->key, KEY_MOTION_THRESHOLD)) {
-                if (entry->value->type == GCONF_VALUE_INT) {
-                        set_motion_threshold (manager, gconf_value_get_int (entry->value));
-                }
-        } else if (! strcmp (entry->key, KEY_TOUCHPAD_DISABLE_W_TYPING)) {
-                if (entry->value->type == GCONF_VALUE_BOOL)
-                        set_disable_w_typing (manager, gconf_value_get_bool (entry->value));
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
-        } else if (! strcmp (entry->key, KEY_TAP_TO_CLICK)) {
-                if (entry->value->type == GCONF_VALUE_BOOL) {
-                        set_tap_to_click (gconf_value_get_bool (entry->value),
-                                          gconf_client_get_bool (client, KEY_LEFT_HANDED, NULL));
-                }
-        } else if (! strcmp (entry->key, KEY_SCROLL_METHOD)) {
-                if (entry->value->type == GCONF_VALUE_INT) {
-                        set_edge_scroll (gconf_value_get_int (entry->value));
-                        set_horiz_scroll (gconf_client_get_bool (client, KEY_PAD_HORIZ_SCROLL, NULL));
-                }
-        } else if (! strcmp (entry->key, KEY_PAD_HORIZ_SCROLL)) {
-                if (entry->value->type == GCONF_VALUE_BOOL)
-                        set_horiz_scroll (gconf_value_get_bool (entry->value));
-#endif
-        } else if (! strcmp (entry->key, KEY_LOCATE_POINTER)) {
-                if (entry->value->type == GCONF_VALUE_BOOL) {
-                        set_locate_pointer (manager, gconf_value_get_bool (entry->value));
-                }
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
-        } else if (! strcmp (entry->key, KEY_TOUCHPAD_ENABLED)) {
-                if (entry->value->type == GCONF_VALUE_BOOL) {
-                    set_touchpad_enabled (gconf_value_get_bool (entry->value));
-                }
-#endif
-        } else if (! strcmp (entry->key, KEY_DWELL_ENABLE)) {
-                if (entry->value->type == GCONF_VALUE_BOOL) {
-                        set_mousetweaks_daemon (manager,
-                                                gconf_value_get_bool (entry->value),
-                                                gconf_client_get_bool (client, KEY_DELAY_ENABLE, NULL));
-                }
-        } else if (! strcmp (entry->key, KEY_DELAY_ENABLE)) {
-                if (entry->value->type == GCONF_VALUE_BOOL) {
-                        set_mousetweaks_daemon (manager,
-                                                gconf_client_get_bool (client, KEY_DWELL_ENABLE, NULL),
-                                                gconf_value_get_bool (entry->value));
-                }
+        switch (g_settings_get_enum (manager->priv->touchpad_settings, KEY_LEFT_HANDED)) {
+        case GSD_TOUCHPAD_HANDEDNESS_RIGHT:
+                return FALSE;
+        case GSD_TOUCHPAD_HANDEDNESS_LEFT:
+                return TRUE;
+        case GSD_TOUCHPAD_HANDEDNESS_MOUSE:
+                return mouse_left_handed;
+        default:
+                g_assert_not_reached ();
         }
 }
 
-static guint
-register_config_callback (GsdMouseManager         *manager,
-                          GConfClient             *client,
-                          const char              *path,
-                          GConfClientNotifyFunc    func)
+static void
+set_mouse_settings (GsdMouseManager *manager)
 {
-        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-        return gconf_client_notify_add (client, path, func, manager, NULL, NULL);
+        gboolean mouse_left_handed, touchpad_left_handed;
+
+        mouse_left_handed = g_settings_get_boolean (manager->priv->mouse_settings, KEY_LEFT_HANDED);
+        touchpad_left_handed = get_touchpad_handedness (manager, mouse_left_handed);
+        set_left_handed (manager, mouse_left_handed, touchpad_left_handed);
+
+        set_motion_acceleration (manager, g_settings_get_double (manager->priv->mouse_settings, KEY_MOTION_ACCELERATION));
+        set_motion_threshold (manager, g_settings_get_int (manager->priv->mouse_settings, KEY_MOTION_THRESHOLD));
+        set_middle_button (manager, g_settings_get_boolean (manager->priv->mouse_settings, KEY_MIDDLE_BUTTON_EMULATION));
+
+        set_disable_w_typing (manager, g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TOUCHPAD_DISABLE_W_TYPING));
+        set_tap_to_click (g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TAP_TO_CLICK), touchpad_left_handed);
+        set_edge_scroll (g_settings_get_enum (manager->priv->touchpad_settings, KEY_SCROLL_METHOD));
+        set_horiz_scroll (g_settings_get_boolean (manager->priv->touchpad_settings, KEY_PAD_HORIZ_SCROLL));
+        set_touchpad_enabled (g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TOUCHPAD_ENABLED));
+}
+
+static void
+mouse_callback (GSettings       *settings,
+                const gchar     *key,
+                GsdMouseManager *manager)
+{
+        if (g_str_equal (key, KEY_DWELL_CLICK_ENABLED) ||
+            g_str_equal (key, KEY_SECONDARY_CLICK_ENABLED)) {
+                set_mousetweaks_daemon (manager,
+                                        g_settings_get_boolean (settings, KEY_DWELL_CLICK_ENABLED),
+                                        g_settings_get_boolean (settings, KEY_SECONDARY_CLICK_ENABLED));
+        } else if (g_str_equal (key, KEY_LOCATE_POINTER)) {
+                set_locate_pointer (manager, g_settings_get_boolean (settings, KEY_LOCATE_POINTER));
+        } else if (g_str_equal (key, KEY_LEFT_HANDED)) {
+                gboolean mouse_left_handed;
+                mouse_left_handed = g_settings_get_boolean (settings, KEY_LEFT_HANDED);
+                set_left_handed (manager, mouse_left_handed, get_touchpad_handedness (manager, mouse_left_handed));
+        } else if (g_str_equal (key, KEY_MOTION_ACCELERATION)) {
+                set_motion_acceleration (manager, g_settings_get_double (settings, KEY_MOTION_ACCELERATION));
+        } else if (g_str_equal (key, KEY_MOTION_THRESHOLD)) {
+                set_motion_threshold (manager, g_settings_get_int (settings, KEY_MOTION_THRESHOLD));
+        } else if (g_str_equal (key, KEY_MIDDLE_BUTTON_EMULATION)) {
+                set_middle_button (manager, g_settings_get_boolean (settings, KEY_MIDDLE_BUTTON_EMULATION));
+        }
+}
+
+static void
+touchpad_callback (GSettings       *settings,
+                   const gchar     *key,
+                   GsdMouseManager *manager)
+{
+        if (g_str_equal (key, KEY_TOUCHPAD_DISABLE_W_TYPING)) {
+                set_disable_w_typing (manager, g_settings_get_boolean (manager->priv->touchpad_settings, key));
+        } else if (g_str_equal (key, KEY_TAP_TO_CLICK)) {
+                set_tap_to_click (g_settings_get_boolean (settings, key),
+                                  g_settings_get_boolean (manager->priv->touchpad_settings, KEY_LEFT_HANDED));
+        } else if (g_str_equal (key, KEY_SCROLL_METHOD)) {
+                set_edge_scroll (g_settings_get_enum (settings, key));
+                set_horiz_scroll (g_settings_get_boolean (settings, KEY_PAD_HORIZ_SCROLL));
+        } else if (g_str_equal (key, KEY_PAD_HORIZ_SCROLL)) {
+                set_horiz_scroll (g_settings_get_boolean (settings, key));
+        } else if (g_str_equal (key, KEY_TOUCHPAD_ENABLED)) {
+                set_touchpad_enabled (g_settings_get_boolean (settings, key));
+        }
+        /* FIXME handle KEY_LEFT_HANDED, KEY_MOTION_ACCELERATION and KEY_MOTION_THRESHOLD */
 }
 
 static void
@@ -997,48 +970,35 @@ gsd_mouse_manager_init (GsdMouseManager *manager)
 static gboolean
 gsd_mouse_manager_idle_cb (GsdMouseManager *manager)
 {
-        GConfClient *client;
-
         gnome_settings_profile_start (NULL);
 
-        client = gconf_client_get_default ();
+        manager->priv->mouse_settings = g_settings_new (SETTINGS_MOUSE_DIR);
+        g_signal_connect (manager->priv->mouse_settings, "changed",
+                          G_CALLBACK (mouse_callback), manager);
+
+        manager->priv->mouse_a11y_settings = g_settings_new ("org.gnome.desktop.a11y.mouse");
+        g_signal_connect (manager->priv->mouse_a11y_settings, "changed",
+                          G_CALLBACK (mouse_callback), manager);
+
+        manager->priv->touchpad_settings = g_settings_new (SETTINGS_TOUCHPAD_DIR);
+        g_signal_connect (manager->priv->touchpad_settings, "changed",
+                          G_CALLBACK (touchpad_callback), manager);
 
-        manager->priv->notify =
-                register_config_callback (manager,
-                                          client,
-                                          GCONF_MOUSE_DIR,
-                                          (GConfClientNotifyFunc) mouse_callback);
-        manager->priv->notify_a11y =
-                register_config_callback (manager,
-                                          client,
-                                          GCONF_MOUSE_A11Y_DIR,
-                                          (GConfClientNotifyFunc) mouse_callback);
-        manager->priv->notify_touchpad =
-                register_config_callback (manager,
-                                          client,
-                                          GCONF_TOUCHPAD_DIR,
-                                          (GConfClientNotifyFunc) mouse_callback);
         manager->priv->syndaemon_spawned = FALSE;
 
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         set_devicepresence_handler (manager);
-#endif
         set_mouse_settings (manager);
-        set_locate_pointer (manager, gconf_client_get_bool (client, KEY_LOCATE_POINTER, NULL));
+        set_locate_pointer (manager, g_settings_get_boolean (manager->priv->mouse_settings, KEY_LOCATE_POINTER));
         set_mousetweaks_daemon (manager,
-                                gconf_client_get_bool (client, KEY_DWELL_ENABLE, NULL),
-                                gconf_client_get_bool (client, KEY_DELAY_ENABLE, NULL));
+                                g_settings_get_boolean (manager->priv->mouse_a11y_settings, KEY_DWELL_CLICK_ENABLED),
+                                g_settings_get_boolean (manager->priv->mouse_a11y_settings, KEY_SECONDARY_CLICK_ENABLED));
 
-        set_disable_w_typing (manager, gconf_client_get_bool (client, KEY_TOUCHPAD_DISABLE_W_TYPING, NULL));
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
-        set_tap_to_click (gconf_client_get_bool (client, KEY_TAP_TO_CLICK, NULL),
-                          gconf_client_get_bool (client, KEY_LEFT_HANDED, NULL));
-        set_edge_scroll (gconf_client_get_int (client, KEY_SCROLL_METHOD, NULL));
-        set_horiz_scroll (gconf_client_get_bool (client, KEY_PAD_HORIZ_SCROLL, NULL));
-        set_touchpad_enabled (gconf_client_get_bool (client, KEY_TOUCHPAD_ENABLED, NULL));
-#endif
-
-        g_object_unref (client);
+        set_disable_w_typing (manager, g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TOUCHPAD_DISABLE_W_TYPING));
+        set_tap_to_click (g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TAP_TO_CLICK),
+                          g_settings_get_boolean (manager->priv->mouse_settings, KEY_LEFT_HANDED));
+        set_edge_scroll (g_settings_get_enum (manager->priv->touchpad_settings, KEY_SCROLL_METHOD));
+        set_horiz_scroll (g_settings_get_boolean (manager->priv->touchpad_settings, KEY_PAD_HORIZ_SCROLL));
+        set_touchpad_enabled (g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TOUCHPAD_ENABLED));
 
         gnome_settings_profile_end (NULL);
 
@@ -1062,37 +1022,21 @@ void
 gsd_mouse_manager_stop (GsdMouseManager *manager)
 {
         GsdMouseManagerPrivate *p = manager->priv;
-        GConfClient *client;
 
         g_debug ("Stopping mouse manager");
 
-        client = gconf_client_get_default ();
-
-        if (p->notify != 0) {
-                gconf_client_remove_dir (client, GCONF_MOUSE_DIR, NULL);
-                gconf_client_notify_remove (client, p->notify);
-                p->notify = 0;
+        if (p->device_manager != NULL) {
+                g_object_unref (p->device_manager);
+                p->device_manager = NULL;
         }
 
-        if (p->notify_a11y != 0) {
-                gconf_client_remove_dir (client, GCONF_MOUSE_A11Y_DIR, NULL);
-                gconf_client_notify_remove (client, p->notify_a11y);
-                p->notify_a11y = 0;
-        }
+        g_object_unref (p->mouse_settings);
+        p->mouse_settings = NULL;
 
-        if (p->notify_touchpad != 0) {
-                gconf_client_remove_dir (client, GCONF_TOUCHPAD_DIR, NULL);
-                gconf_client_notify_remove (client, p->notify_touchpad);
-                p->notify_touchpad = 0;
-        }
-
-        g_object_unref (client);
+        g_object_unref (p->touchpad_settings);
+        p->touchpad_settings = NULL;
 
         set_locate_pointer (manager, FALSE);
-
-#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
-        gdk_window_remove_filter (NULL, devicepresence_filter, manager);
-#endif
 }
 
 static void
@@ -1107,6 +1051,15 @@ gsd_mouse_manager_finalize (GObject *object)
 
         g_return_if_fail (mouse_manager->priv != NULL);
 
+        if (mouse_manager->priv->mouse_settings != NULL)
+                g_object_unref (mouse_manager->priv->mouse_settings);
+
+        if (mouse_manager->priv->mouse_a11y_settings != NULL)
+                g_object_unref (mouse_manager->priv->mouse_a11y_settings);
+
+        if (mouse_manager->priv->touchpad_settings != NULL)
+                g_object_unref (mouse_manager->priv->touchpad_settings);
+
         G_OBJECT_CLASS (gsd_mouse_manager_parent_class)->finalize (object);
 }
 
